<?php
/**
 * 2015-2017 Bonpresta
 *
 * Bonpresta QR code
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the General Public License (GPL 2.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/GPL-2.0
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade the module to newer
 * versions in the future.
 *
 *  @author    Bonpresta
 *  @copyright 2015-2017 Bonpresta
 *  @license   http://opensource.org/licenses/GPL-2.0 General Public License (GPL 2.0)
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Preloading extends Module
{
    public function __construct()
    {
        $this->name = 'preloading';
        $this->tab = 'front_office_features';
        $this->version = '1.0.0';
        $this->bootstrap = true;
        $this->author = 'Bonpresta';
        $this->module_key = 'a1f9610b047c03f66c5ce6f53fcf08e5';
        parent::__construct();
        $this->default_language = Language::getLanguage(Configuration::get('PS_LANG_DEFAULT'));
        $this->id_shop = Context::getContext()->shop->id;
        $this->displayName = $this->l('Preloader');
        $this->description = $this->l('Display preloader.');
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->img_dir = __PS_BASE_URI__. 'modules/preloading/views/img/';
    }

    protected function getConfigurations()
    {
        $configurations = array(
            'PRELOADER_ITEM_STATUS' => true,
            'PRELOADER_ITEM_BACKGROUND' => '#ffffff',
            'PRELOADER_ITEM_STYLE' => 'loader',
            'PRELOADER_ITEM_IMAGE' => $this->img_dir . 'preloader.gif'
        );

        return $configurations;
    }

    public function install()
    {
        $this->clearCache();

        $configurations = $this->getConfigurations();

        foreach ($configurations as $name => $config) {
            Configuration::updateValue($name, $config);
        }

        return parent::install() &&
        $this->registerHook('displayBackOfficeHeader') &&
        $this->registerHook('displayHeader');
    }

    public function uninstall()
    {
        $configurations = $this->getConfigurations();

        foreach (array_keys($configurations) as $config) {
            Configuration::deleteByName($config);
        }

        return parent::uninstall();
    }

    public function getContent()
    {
        $output = '';
        $result = '';

        if ((bool)Tools::isSubmit('submitSettings')) {
            if (!$result = $this->preValidateForm()) {
                $this->processImageUpload($_FILES);
                $output .= $this->postProcess();
                $this->clearCache();
                $output .= $this->displayConfirmation($this->l('Save all settings.'));
            } else {
                $output = $result;
                $output .= $this->renderTabForm();
            }
        }

        if (!$result) {
            $output .= $this->renderTabForm();
        }

        return $output;
    }

    protected function renderTabForm()
    {
        $loader = '';
        $loader .= $this->l('Here you can upload images for type: "Custom image"').'<br/><img class="img_preloader" alt="" src="'. $this->img_dir . 'preloader.gif" width="100" height="100" /><br />';

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable'),
                        'name' => 'PRELOADER_ITEM_STATUS',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'enable',
                                'value' => 1,
                                'label' => $this->l('Yes')),
                            array(
                                'id' => 'disable',
                                'value' => 0,
                                'label' => $this->l('No')),
                        ),
                    ),
                    array(
                        'type' => 'color',
                        'label' => $this->l('Background:'),
                        'name' => 'PRELOADER_ITEM_BACKGROUND',
                        'desc' => $this->l('Background for display loader.'),
                    ),
                    array(
                        'type' => 'radio',
                        'label' => $this->l('Type loader:'),
                        'name' => 'PRELOADER_ITEM_STYLE',
                        'values' => array(
                            array(
                                'id' => 'loader',
                                'value' => 'loader',
                                'label' => $this->l('Css animation'),
                                'img_link' => $this->img_dir . '/custom_loader.png'
                            ),
                            array(
                                'id' => 'logo',
                                'value' => 'logo',
                                'label' => $this->l('Your logo'),
                                'img_link' => $this->img_dir . '/logo.png'
                            ),
                            array(
                                'id' => 'custom_loader',
                                'value' => 'custom_loader',
                                'label' => $this->l('Custom image'),
                                'img_link' => $this->img_dir . '/preloader.gif'
                            ),

                        ),
                    ),
                    array(
                        'type' => 'file',
                        'label' => $this->l('Preloader:'),
                        'name' => 'PRELOADER_ITEM_IMAGE',
                        'desc' => $loader,
                        'display_image' => true,
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitSettings';
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues()
    {
        $fields = array();
        $configurations = $this->getConfigurations();

        foreach (array_keys($configurations) as $config) {
            $fields[$config] = Configuration::get($config);
        }

        return $fields;
    }

    protected function postProcess()
    {
        $form_values = $this->getConfigFieldsValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    public function processImageUpload($FILES)
    {
        if (isset($FILES['PRELOADER_ITEM_IMAGE']) && isset($FILES['PRELOADER_ITEM_IMAGE']['tmp_name']) && !empty($FILES['PRELOADER_ITEM_IMAGE']['tmp_name'])) {
            if (ImageManager::validateUpload($FILES['PRELOADER_ITEM_IMAGE'], 400000)
                || ImageManager::getMimeTypeByExtension($FILES['PRELOADER_ITEM_IMAGE']['name']) != 'image/gif') {
                $this->_errors[] = $this->l('Invalid image type or size. Maximum image width and height equal 300px');
            } else {
                $ext = Tools::substr($FILES['PRELOADER_ITEM_IMAGE']['name'], strrpos($FILES['PRELOADER_ITEM_IMAGE']['name'], '.') + 1);
                $file_name = 'preloader.' . $ext;
                $path = _PS_MODULE_DIR_ .'preloading/views/img/' . $file_name;
                if (!move_uploaded_file($FILES['PRELOADER_ITEM_IMAGE']['tmp_name'], $path)) {
                    return $this->_errors[] = $this->l('An error occurred while attempting to upload the file.');
                } else {
                    Configuration::updateValue('PRELOADER_ITEM_IMAGE', $this->img_dir.$file_name);
                }
            }
        }
    }

    protected function preValidateForm()
    {
        $errors = array();
        $image = Tools::getValue('PRELOADER_ITEM_IMAGE');
        $type = Tools::strtolower(Tools::substr(strrchr($image, '.'), 1));

        if (!empty($image) && $type != 'gif'){
            $errors[] = $this->l('Image format not recognized, allowed format is: .gif');
        }

        if (!Validate::isColor(Tools::getValue('PRELOADER_ITEM_BACKGROUND'))) {
            $errors[] = $this->l('"Background" format error.');
        }

        if (count($errors)) {
            return $this->displayError(implode('<br />', $errors));
        }

        return false;
    }

    protected function getStringValueType($data)
    {
        if (Validate::isInt($data)) {
            return 'int';
        } elseif (Validate::isFloat($data)) {
            return 'float';
        } elseif (Validate::isBool($data)) {
            return 'bool';
        } else {
            return 'string';
        }
    }

    protected function getSettings()
    {
        $settings = $this->getConfigurations();
        $get_settings = array();

        foreach (array_keys($settings) as $name) {
            $data = Configuration::get($name);
            $get_settings[$name] = array('value' => $data, 'type' => $this->getStringValueType($data));
        }

        return $get_settings;
    }

    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('configure') != $this->name) {
            return;
        }

        $this->context->controller->addJquery();
        $this->context->controller->addCSS($this->_path.'views/css/preloading_admin.css');
    }

    protected function clearCache()
    {
        $this->_clearCache('preloading-header.tpl');
    }

    public function hookDisplayHeader()
    {
        if (Configuration::get('PRELOADER_ITEM_STATUS')) {
            Media::addJsDef(array('logo_url' => _PS_IMG_.Configuration::get('PS_LOGO')));
            Media::addJsDef(array('preloader_image' => $this->img_dir . 'preloader.gif'));

            if (Configuration::get('PRELOADER_ITEM_STYLE') == 'loader') {
                $this->context->controller->addCSS($this->_path.'views/css/preloading.css', 'all');
            } elseif (Configuration::get('PRELOADER_ITEM_STYLE') == 'custom_loader') {
                $this->context->controller->addCSS($this->_path.'views/css/preloading_custom_image.css', 'all');
            } elseif (Configuration::get('PRELOADER_ITEM_STYLE') == 'logo') {
                $this->context->controller->addCSS($this->_path.'views/css/preloading_logo.css', 'all');
            }

            $this->context->controller->addJS($this->_path.'/views/js/preloading.js');
            if (!$this->isCached('preloading-header.tpl', $this->getCacheId())) {
                $this->context->smarty->assign('settings', $this->getSettings());
            }

            return $this->display($this->_path, '/views/templates/hook/preloading-header.tpl', $this->getCacheId());
        }
    }
}
